PROGRAM CHPF; {Copyright 1997 Paul Milenkovic ALL RIGHTS RESERVED}

USES StrBlock, StdIO;

CONST
    wrk_max = 8192;
    wrk_mrk = 4096;

VAR
    i: INTEGER;
    wrk_buf, out_buf: ARRAY [1 .. wrk_max] OF INTEGER;
    sample_ms, window_ms: DOUBLE;


PROCEDURE HiPassFilter(window_ms, sample_ms: DOUBLE);

    CONST nf = 3;

    VAR mval, nval, nread, m1k, k: INTEGER;
        wrk_len, wrk_dif, wrk_pos, wrk_pre, wrk_pst: INTEGER;
        out_len, out_pos: INTEGER;
        sig, temp, dtheta, y0: DOUBLE;
        yr, yi, gr, gi, wr, wi: ARRAY [1 .. nf] OF DOUBLE;

BEGIN
    mval     := Round(window_ms/(2*sample_ms));
    nval     := Succ(2*mval); {Odd order filter N = 2M+1}
    y0       := 0;  {DC frequency sample}
    m1k      := 1;
    dtheta   := 2*Pi/nval;
    FOR k := 1 TO nf DO BEGIN {Initialize frequency sample oscillators}
        yr[k] := 0;
        yi[k] := 0;
        m1k   := -m1k;
        gr[k] := m1k*2*Cos(0.5*k*dtheta);
        gi[k] := m1k*2*Sin(0.5*k*dtheta);
        wr[k] := Cos(k*dtheta);
        wi[k] := Sin(k*dtheta);
    END;
    wrk_len  := 0;
    wrk_pre  := 0;
    wrk_pos  := 0;
    wrk_pst  := 0;

    REPEAT
        IF wrk_len > wrk_mrk THEN BEGIN
            wrk_dif := wrk_len - wrk_mrk;
            Move(wrk_buf[Succ(wrk_dif)],wrk_buf[1],
                wrk_mrk*Sizeof(wrk_buf[1]));
            wrk_len := wrk_len - wrk_dif;
            wrk_pre := wrk_pre - wrk_dif;
            wrk_pos := wrk_pos - wrk_dif;
            wrk_pst := wrk_pst - wrk_dif;
        END;
        nread := StdBlockRead(
            wrk_buf[Succ(wrk_len)],Sizeof(wrk_buf[1])*(wrk_max-wrk_len));
        wrk_len := wrk_len + nread DIV Sizeof(wrk_buf[1]);

        out_len := wrk_len - wrk_pos;
        IF nread > 0 THEN out_len := out_len - mval;
        FOR out_pos := 1 TO out_len DO BEGIN
            Inc(wrk_pos);
            WHILE wrk_pst < wrk_pos+mval DO BEGIN
                Inc(wrk_pst);
                FOR k := 1 TO nf DO BEGIN
                    temp  := wr[k]*yr[k] - wi[k]*yi[k];
                    yi[k] := wi[k]*yr[k] + wr[k]*yi[k];
                    yr[k] := temp;
                END;
                IF wrk_pst <= wrk_len THEN BEGIN
                    sig := wrk_buf[wrk_pst];
                    y0 := y0 + sig;
                    FOR k := 1 TO nf DO BEGIN
                        yr[k] := yr[k] + gr[k]*sig;
                        yi[k] := yi[k] + gi[k]*sig;
                    END;
                END;
            END;
            IF wrk_pre < Pred(wrk_pos-mval) THEN BEGIN
                Inc(wrk_pre);
                sig := wrk_buf[wrk_pre];
                y0 := y0 - sig;
                FOR k := 1 TO nf DO BEGIN
                    yr[k] := yr[k] - gr[k]*sig;
                    yi[k] := yi[k] - gi[k]*sig;
                END;
            END;
            out_buf[out_pos] :=  wrk_buf[wrk_pos] -
                Round((y0+0.989*yr[1]+0.69*yr[2]+0.09*yr[3])/nval);
        END;
        StdBlockWrite(out_buf,out_len*Sizeof(out_buf[1]));
    UNTIL nread = 0;
END;

BEGIN
    StdSetErrorExit;
    window_ms := 0;
    FOR i := 1 TO ParamCount DO BEGIN
        UpdateDoubleKeyword(window_ms,ParamStr(i),'/W:');
    END;
    StdCmd('input');
    StdCmd('output');
    sample_ms   := StdDbl('sample_ms?');
    IF window_ms <= 0 THEN
        window_ms := StdDbl('DOUBLE:Window length (ms)');
    IF (window_ms > 0) AND (sample_ms > 0) THEN BEGIN
        StdCmd('enable_break');
        StdCmd('');
        HiPassFilter(window_ms,sample_ms);
    END;
END.
