PROGRAM CZCF; {Copyright 1997 Paul Milenkovic ALL RIGHTS RESERVED}

USES StrBlock, StdIO;

CONST
    wrk_max = 8192;
    wrk_mrk = 4096;

VAR
    i: INTEGER;
    wrk_buf, out_buf: ARRAY [1 .. wrk_max] OF INTEGER;
    sample_ms, window_ms, buf_ms, range, scale, output_scale: DOUBLE;
    voiced, unvoiced: BOOLEAN;
    gname: STRING;
    gfile: TEXT;


PROCEDURE ZCF(window_ms, sample_ms: DOUBLE);

    VAR mval, nval, nread, k, nzc: INTEGER;
        wrk_len, wrk_dif, wrk_pos, wrk_pre, wrk_pst: INTEGER;
        out_len, out_uv: INTEGER;
        sig0, sumsq, scale_db, scale_cf, db_scale: SINGLE;
        avg_cf, avg_db: SINGLE;
        v_ms, uv_ms: DOUBLE;
        vu_count, vu_max: INTEGER;

BEGIN
    vu_max   := Round(1.0/sample_ms);
    vu_count := 0;
    mval     := Round(window_ms/(2*sample_ms));
    nval     := Succ(2*mval); {Odd order filter N = 2M+1}
    nzc      := 0;
    wrk_len  := 0;
    wrk_pre  := 0;
    wrk_pos  := 0;
    wrk_pst  := 0;
    out_uv   := 0;
    sumsq    := 0;
    scale_db := scale*scale/nval;
    db_scale := 10.0/Ln(10.0);
    scale_cf := 0.5/(sample_ms*nval);
    v_ms     := 0;
    uv_ms    := 0;

    REPEAT
        IF wrk_len > wrk_mrk THEN BEGIN
            wrk_dif := wrk_len - wrk_mrk;
            IF (NOT voiced) AND (NOT unvoiced) THEN
                StdBlockWrite(out_buf[1],wrk_dif*Sizeof(INTEGER));
            Move(out_buf[Succ(wrk_dif)],out_buf[1],
                wrk_mrk*Sizeof(out_buf[1]));
            Move(wrk_buf[Succ(wrk_dif)],wrk_buf[1],
                wrk_mrk*Sizeof(wrk_buf[1]));
            buf_ms := buf_ms + wrk_dif*sample_ms;
            Dec(wrk_len,wrk_dif);
            Dec(wrk_pre,wrk_dif);
            Dec(wrk_pos,wrk_dif);
            Dec(wrk_pst,wrk_dif);
            Dec(out_uv, wrk_dif);
            IF out_uv < 0 THEN
                out_uv := 0;
        END;
        nread := StdBlockRead(
            wrk_buf[Succ(wrk_len)],Sizeof(wrk_buf[1])*(wrk_max-wrk_len));
        wrk_len := wrk_len + nread DIV Sizeof(wrk_buf[1]);

        out_len := wrk_len;
        IF nread > 0 THEN
            out_len := out_len - mval;
        WHILE wrk_pos < out_len DO BEGIN
            Inc(wrk_pos);
            WHILE wrk_pst < wrk_pos+mval DO BEGIN
                Inc(wrk_pst);
                IF (wrk_pst <= wrk_len) AND (wrk_pst > 1) THEN BEGIN
                    IF (wrk_buf[wrk_pst] < 0) XOR
                       (wrk_buf[Pred(wrk_pst)] < 0)
                    THEN
                        Inc(nzc);
                    sig0 := wrk_buf[wrk_pst];
                    sumsq := sumsq + sig0*sig0;
                END;
            END;
            IF wrk_pre < Pred(wrk_pos-mval) THEN BEGIN
                Inc(wrk_pre);
                IF wrk_pre > 1 THEN BEGIN
                    IF (wrk_buf[wrk_pre] < 0) XOR
                       (wrk_buf[Pred(wrk_pre)] < 0)
                    THEN
                        Dec(nzc);
                    sig0 := wrk_buf[wrk_pre];
                    sumsq := sumsq - sig0*sig0;
                END;
            END;
            avg_cf := scale_cf*nzc;
            avg_db := db_scale*Ln(scale_db*sumsq);
{            IF (avg_db <= -40) OR
               (avg_cf >= 2.0 - 1.0*avg_db/(-40))
            THEN BEGIN
 }               IF wrk_pos - out_uv < 2*nval THEN
                    FOR i := Succ(out_uv) TO Pred(wrk_pos) DO
                        out_buf[i] := Round(output_scale*avg_cf);
                out_buf[wrk_pos] := Round(output_scale*avg_cf);
                out_uv := wrk_pos;
{            END ELSE
                out_buf[wrk_pos] := 0;
 }           Inc(vu_count);
            IF (voiced OR unvoiced) AND
               (vu_count = vu_max) AND
               (buf_ms+Pred(wrk_pos)*sample_ms > uv_ms) AND
               (NOT EOF(gfile))
            THEN
                Readln(gfile,v_ms,uv_ms);
            IF (unvoiced AND
                (vu_count = vu_max) AND
                ((buf_ms+Pred(wrk_pos)*sample_ms < v_ms) OR
                 (buf_ms+Pred(wrk_pos)*sample_ms > uv_ms))) OR
               (voiced AND
                (vu_count = vu_max) AND
                (buf_ms+Pred(wrk_pos)*sample_ms >= v_ms) AND
                (buf_ms+Pred(wrk_pos)*sample_ms < uv_ms))
            THEN
                Writeln(avg_cf:10:3,avg_db:10:3);
            IF vu_count = vu_max THEN vu_count := 0;
        END;
    UNTIL nread = 0;

    IF (NOT voiced) AND (NOT unvoiced) THEN BEGIN
        IF wrk_len > 0 THEN
            StdBlockWrite(out_buf[1],wrk_len*Sizeof(INTEGER));
        StdSuspendWrite;
    END;
END;

BEGIN
    StdSetErrorExit;

    window_ms  := 0;
    voiced     := FALSE;
    unvoiced   := FALSE;
    FOR i := 1 TO ParamCount DO BEGIN
        UpdateDoubleKeyword(window_ms,ParamStr(i),'/W:');
        IF ParamStr(i) = '/V' THEN voiced := TRUE;
        IF ParamStr(i) = '/U' THEN unvoiced := TRUE;
    END;

    StdCmd('leader');
    StdCmd('extension=lbv');
    StdCmd('input');
    IF voiced OR unvoiced THEN BEGIN
        gname := StdTxt('aname?');
        Assign(gfile,gname);
        {$I-} Reset(gfile) {$I+};
        IF IOResult <> 0 THEN BEGIN
            voiced := FALSE;
            unvoiced := FALSE;
        END;
    END ELSE
        StdCmd('output');

    sample_ms   := StdDbl('sample_ms?');
    IF window_ms <= 0 THEN
        window_ms := StdDbl('DOUBLE:Window length (ms)');
    IF (window_ms > 0) AND (sample_ms > 0) THEN BEGIN
        range := 1000.0/sample_ms;
        IF StdInt('RES?') = 16 THEN
            range := range * 2;
        StdSetDbl('range=',range,9);
        StdCmd('offset=0.0');
        output_scale := 1000.0/StdDbl('data_scale?'); {1000.0 for Hz-kHz}
        scale := StdDbl('scale?');
        buf_ms := StdDbl('start_ms?');
        StdCmd('units_name=Hz');
        StdCmd('enable_break');
        StdCmd('');
        ZCF(window_ms,sample_ms);
        IF voiced OR unvoiced THEN
            Close(gfile);
    END;
END.
