PROGRAM FricGM; {Copyright 1999 Paul Milenkovic ALL RIGHTS RESERVED}


USES StrBlock, StdIO, DSPTypes, DSPFast, FFT, Moments;

CONST
    wrk_max = 8192;
    wrk_mrk = 4096;
    mbuffr  = 1024;


VAR fname, extension: ASCIIZ_string;
    sample_ms: DOUBLE;
    data_scale, gain: SINGLE;
    wrk_buf: ARRAY [1 .. wrk_max] OF INTEGER;


PROCEDURE ScanCommandLineParameters;

    VAR i: INTEGER;
        pstr: ASCIIZ_string;

BEGIN
    fname     := '';
    extension := '';
    gain      := 1.0;

    FOR i := 1 TO ParamCount DO BEGIN
        pstr := ParamStr(i);
        UpdateSingleKeyword(gain,ParamStr(i),'/G:');
        UpdateStringKeyword(extension,ParamStr(i),'.');
        IF (fname = '')     AND
           (extension = '') AND
           (Copy(pstr,1,1) <> '/')
        THEN
            fname := pstr;
    END;
END;


PROCEDURE ComputeCoef(VAR b0, a1, a2: SINGLE; n: INTEGER);

    VAR y0, y1, y2: SINGLE;
        i: INTEGER;

BEGIN
    a2 := Exp(-Pi*0.100*sample_ms);
    a1 := -2*a2*Cos(2*Pi*0.050*sample_ms);
    a2 := a2*a2;
    b0 := 1;
{
    y2 := -a1;
    y1 := -1 + a1*a1 - a2;
 }
{
    y2 := -a1 - 1;
    y1 := a1*a1 + a1 - a2;
    b0 := 1 + y2*y2 + y1*y1;
    FOR i := 3 TO Pred(n) DO BEGIN
        y0 := - a1*y1 - a2*y2;
        y2 := y1;
        y1 := y0;
        b0 := b0 + y0*y0;
    END;
    IF b0 > 0 THEN b0 := 1/Sqrt(b0);
 }
END;


PROCEDURE InputFrication;

    VAR vfile: TEXT;
        wrk_len, wrk_dif, nread, ncoef, i, n1, n2, n, k: INTEGER;
        t0, tf, tfric, tfp: DOUBLE;
        afric, x0, x1, x2, y0, y1, y2, b0, a1, a2: SINGLE;
        mean, stdv, skew, kurt: Double;
        frec: fourier_record;
        fmag: fourier_mag;

BEGIN
    Assign(vfile,fname);
    {$I-} Reset(vfile) {$I+};
    IF IOResult = 0 THEN BEGIN
        StdCmd('enable_break');
        ComputeCoef(b0,a1,a2,1024);
        t0 := StdDbl('start_ms?');
        tf := StdDbl('stop_ms?');
        tfric := t0 - 20.0;
        IF EOF(vfile) THEN
            tfp := tf + 20.0
        ELSE
            Read(vfile,tfp);
        wrk_len := 0;
        Randomize;
        StdCmd('enable_break');
        StdCmd('');               {Initiate binary waveform I/O}
        REPEAT
            IF wrk_len > wrk_mrk THEN BEGIN
                wrk_dif := wrk_len - wrk_mrk;
                StdBlockWrite(wrk_buf[1],wrk_dif*Sizeof(INTEGER));
                Move(wrk_buf[Succ(wrk_dif)],wrk_buf[1],
                    wrk_mrk*Sizeof(INTEGER));
                t0 := t0 + wrk_dif * sample_ms;
                Dec(wrk_len,wrk_dif);
            END;
            nread := StdBlockRead(
                wrk_buf[Succ(wrk_len)],
                Sizeof(wrk_buf)-Sizeof(INTEGER)*wrk_len
            );
            wrk_len := wrk_len + nread DIV Sizeof(INTEGER);

            WHILE (tfric <= tf) AND
                  ((tfric <= t0 + Pred(wrk_len-mbuffr)*sample_ms)
                   OR (nread = 0))
            DO BEGIN
                IF tfric >= t0 THEN BEGIN
                    n1 := Succ(Round((tfric - 5.0 - t0)/sample_ms));
                    n2 := Succ(Round((tfric + 5.0 - t0)/sample_ms));
                    x2 := 0;
                    x1 := 0;
                    y2 := 0;
                    y1 := 0;
                    n := n1-50;
                    IF n < 1 THEN n := 1;

                    for i := 0 to 1023 do frec.s[i] := 0;
                    for i := 50 to 50+n2-n1 do
                        frec.s[i] := afric*data_scale*gain*
                            Sin(Pi*(i-50)/(n2-n1))*(Random - 0.5)*Sqrt(12);
                    MomentFilt(frec,10,fmag,512);

                    WHILE (n <= n2+50) AND (n <= wrk_len) DO BEGIN
                        x0 := frec.s[50+n-n1];
                        y0 := b0*(x0 - x1) - a1*y1 - a2*y2;
                        x2 := x1;
                        x1 := x0;
                        y2 := y1;
                        y1 := y0;
                        wrk_buf[n] := wrk_buf[n] + Round(y0);
                        Inc(n);
                    END;
                    WHILE (n <= n2+mbuffr) AND (n <= wrk_len) DO BEGIN
                        y0 := -b0*x2 - a1*y1 - a2*y2;
                        x2 := x1;
                        x1 := 0;
                        y2 := y1;
                        y1 := y0;
                        wrk_buf[n] := wrk_buf[n] + Round(y0);
                        Inc(n);
                    END;
                END;
                tfric := tfp;
                IF tfric <= tf THEN BEGIN
                    Readln(vfile,afric,mean,stdv,skew,kurt);
                    MomentModel(mean,stdv,skew,kurt,fmag,512,sample_ms);
                END;
                IF EOF(vfile) THEN
                    tfp := tf + 20.0
                ELSE
                    Read(vfile,tfp);
            END;
        UNTIL nread = 0;

        IF wrk_len > 0 THEN
            StdBlockWrite(wrk_buf,wrk_len*Sizeof(INTEGER));
        StdSuspendWrite;
        Close(vfile);
    END;
END;

BEGIN
    StdSetErrorExit;
    ScanCommandLineParameters;

    StdCmd(Concat('extension=',extension));
    StdCmd('input');
    StdCmd('output');
    IF fname = '' THEN BEGIN
        IF extension = '' THEN
            fname := StdTxt('ename?')
        ELSE
            fname := StdTxt('fname?');
    END;
    sample_ms := StdDbl('sample_ms?');

    IF (fname <> '') AND (sample_ms > 0) THEN BEGIN
        data_scale := 1.0/StdDbl('data_scale?');
        InputFrication;
    END;
END.
