PROGRAM FricMm; {Copyright 1999 Paul Milenkovic ALL RIGHTS RESERVED}


USES StrBlock, StdIO, DSPTypes, DSPFast, FFT, Moments, FiltBuf;

CONST
    window_ms: DOUBLE = 20.0;
    time_step: DOUBLE =  5.0;

VAR
    sample_ms, start_ms, stop_ms: DOUBLE;
    ncoef, i: INTEGER;
    scale, alpha, data_scale: DOUBLE;

    pstr, extension, gname, oname: ASCIIZ_string;
    gfile, ofile: TEXT;

    wrk_buf: FiltBuf.wrk_bufT;


PROCEDURE MomentAnalysis;

    VAR
        nread, npoint, np, i, n: INTEGER;

        wrk_len, wrk_dif, wrk_pos: INTEGER;

        sig: DSPTypes.sig_buffer;
        frec: fourier_record;
        fmag: fourier_mag;
        tref, tpos, tpm, tp: DOUBLE;

        avg_db, avg_cf, rescale, samp, rms: SINGLE;
        mean, stdv, skew, kurt: DOUBLE;

    PROCEDURE UpdateSamplePointer;

    BEGIN
        tref := tref + time_step;
        wrk_dif := Round((tref - tpos)/sample_ms);
        wrk_pos := wrk_pos + wrk_dif;
        tpos := tpos + sample_ms*wrk_dif;
    END;

BEGIN
    npoint  := Round(window_ms/sample_ms);
    IF npoint > mbuf THEN
        npoint := mbuf;
    wrk_len := 0;
    wrk_pos := 1 - Pred(npoint DIV 2);
    tpos := start_ms;
    tref := tpos;
    tp := start_ms - window_ms;

    WHILE wrk_pos < 2 DO
        UpdateSamplePointer;

    REPEAT
        IF wrk_len > wrk_mrk THEN BEGIN
            wrk_dif := wrk_len - wrk_mrk;
            Move(wrk_buf[Succ(wrk_dif)],wrk_buf[1],wrk_mrk*Sizeof(INTEGER));
            Dec(wrk_len,wrk_dif);
            Dec(wrk_pos,wrk_dif);
        END;
        nread := StdBlockRead(
            wrk_buf[Succ(wrk_len)],Sizeof(wrk_buf)-Sizeof(INTEGER)*wrk_len
        );
        wrk_len := wrk_len + nread DIV Sizeof(INTEGER);

        WHILE Pred(wrk_pos + npoint) <= wrk_len DO BEGIN
            IF tref > tp - 0.5*window_ms + 0.5*time_step THEN BEGIN
                REPEAT
                    tpm := tp;
                    IF EOF(gfile) THEN
                        tp := stop_ms + window_ms + time_step
                    ELSE
                        Readln(gfile,tp);
                UNTIL (tpos <= tp) OR EOF(gfile);
                tref := tp - 0.5*window_ms;
                WHILE tref > tpos - 0.5*time_step DO
                    tref := tref - time_step;
            END ELSE
            IF tref > tpm + 0.25*window_ms THEN BEGIN
                IF tpos > tp - 0.75*window_ms THEN
                    np := Round((tp - tpos + 0.25*window_ms)/sample_ms)
                ELSE
                    np := npoint;
                avg_db := RMSdB(wrk_buf,wrk_pos,np,scale);
                IF avg_db > -90 THEN BEGIN
                    LoadBufferValuesDif(wrk_buf,sig,wrk_pos,np,scale,alpha);

                    for i := 0 to 1023 do frec.s[i] := 0.0;
                    for i := 0 to Pred(np) do
                        frec.s[i] := sig[i]*
                        (0.54 - 0.46*Cos(2*Pi*i/Pred(np)));
                    RealFourierTransform(frec,10);
                    for i := 0 to 512 do with frec.a[i] do
                        fmag[i] := re*re + im*im;
                    ComputeMoments(fmag,512,sample_ms,mean,stdv,skew,kurt);

                    rescale := data_scale/scale;
                    rms := 0.0;
                    n   := 0;
                    FOR i := npoint DIV 4 TO (3*npoint) DIV 4 DO BEGIN
                        samp := rescale*sig[i];
                        rms := rms + samp*samp;
                        Inc(n);
                    END;
                    IF n > 0 THEN rms := rms / n;
                    rms := Sqrt(rms);

                    Writeln(ofile,tpos:0:3,#9,rms:0:6,
                        #9,mean:0:3,#9,stdv:0:3,#9,skew:0:3,#9,kurt:0:3);
                END;
            END;

            UpdateSamplePointer;
        END;
    UNTIL nread = 0;
END;


BEGIN
    StdSetErrorExit;

    gname      := '';
    extension  := '';

    FOR i := 1 TO ParamCount DO BEGIN
        pstr := ParamStr(i);
        UpdateStringKeyword(extension,pstr,'.');
        IF (gname = '')     AND
           (extension = '') AND
           (Copy(pstr,1,1) <> '/')
        THEN
            gname := pstr;
    END;

    StdCmd('leader');
    IF extension <> '' THEN
        StdCmd(Concat('extension=',extension));
    StdCmd('input');
    IF (gname = '') AND (extension <> '') THEN
        gname := StdTxt('aname?');
    StdCmd('extension=fmm');
    oname := StdTxt('fname?');
    IF gname = '' THEN
        gname := StdTxt('ename?');

    sample_ms := StdDbl('sample_ms?');
    IF (sample_ms > 0) AND (gname <> '') AND (oname <> '') THEN BEGIN
        start_ms   := StdDbl('start_ms?');
        stop_ms    := StdDbl('stop_ms?');
        scale      := StdDbl('scale?');
        data_scale := StdDbl('data_scale?');
        ncoef      := StdInt('ncoef?');
        alpha      := StdDbl('alpha?');

        Assign(gfile,gname);
        {$I-} Reset(gfile); {$I+}
        IF IOResult = 0 THEN BEGIN
            Assign(ofile,oname);
            {$I-} Rewrite(ofile); {$I+}
            IF IOResult = 0 THEN BEGIN
                StdCmd('enable_break');
                StdCmd('');
                MomentAnalysis;
                Close(ofile);
            END;
            Close(gfile);
        END;
    END;
END.