UNIT StdIO; {Copyright 1990, 1997 Paul Milenkovic ALL RIGHTS RESERVED}

INTERFACE USES DOS;


PROCEDURE StdCmd(cmd: String);
    {Sends command string to filter interface}

PROCEDURE StdSetDbl(cmd: String; dval: Double; precision: Integer);
    {Sends command string with appended alphanumeric dval}

PROCEDURE StdSetInt(cmd: String; ival: Integer);
    {Sends command string withs alphanumeric ival}


FUNCTION  StdTxt(cmd: String): String;
    {Receives string response from filter interface}

FUNCTION  StdDbl(cmd: String): Double;
    {Receives alphanumeric string and returns as double-precision float}

FUNCTION  StdInt(cmd: String): Integer;
    {Receives alphanumeric string and returns as integer}


FUNCTION  StdBlockRead(VAR buf; count: Integer): Integer;
    {Reads count -bytes- worth of integer samples from filter interface -
     count should be multiple of 2 (number of bytes in and Integer) -
     returns byte count of samples actually read}

PROCEDURE StdBlockWrite(VAR buf; count: Integer);
    {Writes count -bytes- of integer samples to filter interface}

FUNCTION  StdNWrite: Integer;
    {Byte count of what was actually written}

PROCEDURE StdSuspendRead;
    {Suspend binary filter I/O for 'input' or 'input' 'output' condition -
     resume binary I/O with  StdCmd('resume_binary_IO')}

PROCEDURE StdSuspendWrite;
    {Suspend binary filter I/O for 'output' condition}


PROCEDURE StdSetErrorExit;
    {Install a handler that displays how your filter terminated under
     divide-by-zero, other type of exception terminations}


IMPLEMENTATION

VAR nwrite: Integer;
    ExitP:  POINTER;

FUNCTION  HexString(num: Word): String;

    VAR sval: String[4];
        i: Integer;
        fld: Word;

BEGIN
    sval := '';
    FOR i := 0 TO 3 DO BEGIN
        fld := num SHR 12;
        num := num SHL 4;
        IF fld > 9 THEN
            sval := Concat(sval,CHAR(55+fld))
        ELSE
            sval := Concat(sval,CHAR(48+fld));
    END;
    HexString := sval;
END;


FUNCTION  IntString(num: Integer): String;

    VAR sval: String;

BEGIN
    Str(num:0,sval);
    IntString := sval;
END;


PROCEDURE StdCmd(cmd: String);

BEGIN
    IF cmd = '' THEN cmd := 'initiate_binary_IO';
    StdBlockWrite(cmd[1],Length(cmd));
END;


PROCEDURE StdSetDbl(cmd: String; dval: Double; precision: Integer);

    VAR dString: String;

BEGIN
    Str(dval:0:precision,dString);
    StdCmd(Concat(cmd,dString));
END;


PROCEDURE StdSetInt(cmd: String; ival: Integer);

BEGIN
    StdCmd(Concat(cmd,IntString(ival)));
END;


FUNCTION  StdStr: String;

    VAR rstr: String;

BEGIN
    rstr[0] := Char(StdBlockRead(rstr[1],128));
    StdStr  := rstr;
END;


FUNCTION  StdDbl(cmd: String): Double;

    VAR err: Integer;
        rval: Double;

BEGIN
    StdCmd(cmd);
    rval := 0;
    Val(StdStr,rval,err);
    StdDbl := rval;
END;


FUNCTION  StdInt(cmd: String): Integer;

BEGIN
    StdInt := Round(StdDbl(cmd));
END;


FUNCTION  StdTxt(cmd: String): String;

BEGIN
    StdCmd(cmd);
    StdTxt := StdStr;
END;


FUNCTION  StdBlockRead(VAR buf; count: Integer): Integer;


    VAR regs: DOS.registers;

BEGIN
    regs.ax := $3F00;
    regs.bx := 0;
    regs.cx := count;
    regs.ds := Seg(buf);
    regs.dx := Ofs(buf);
    Intr($21,regs);
    StdBlockRead := regs.ax;
END;


PROCEDURE StdBlockWrite(VAR buf; count: Integer);

    VAR regs: DOS.registers;

BEGIN
    regs.ax := $4000;
    regs.bx := 1;
    regs.cx := count;
    regs.ds := Seg(buf);
    regs.dx := Ofs(buf);
    Intr($21,regs);
    nwrite  := regs.ax;
END;


FUNCTION  StdNWrite: Integer;

BEGIN
    StdNWrite := nwrite;
END;


PROCEDURE StdSuspendRead;

    VAR dummy: ARRAY [1 .. 3] OF Integer;
        i: Integer;

BEGIN
    i := StdBlockRead(dummy,3);
END;


PROCEDURE StdSuspendWrite;

    CONST dummy: ARRAY [1 .. 3] OF BYTE = (0, 0, 0);

BEGIN
    StdBlockWrite(dummy,3);
END;


PROCEDURE ErrorExit;

BEGIN
    IF ErrorAddr <> nil THEN BEGIN
        StdSuspendRead;
        StdSuspendWrite;
        StdCmd(Concat(
            'error_cmd= code = ',IntString(ExitCode),
            ' addr = ',HexString(Seg(ErrorAddr^)),':',
            HexString(Ofs(ErrorAddr^))));
    END;
    ExitCode  := 0;
    ErrorAddr := nil;
    ExitProc  := ExitP;
END;


PROCEDURE StdSetErrorExit;

BEGIN
    ExitP    := ExitProc;
    ExitProc := @ErrorExit;
END;

END.
