unit SVD; {Copyright 1987 Paul Milenkovic ALL RIGHTS RESERVED}

interface

const
    mdata = 5;           {maximum number of data samples in SVD analysis}
    mcomp = 5;           {maximum number of data components}

type
    z_index  = array [1 .. mcomp] of Integer;
    z_array  = array [1 .. mcomp] of Double;
    a0_array = array [1 .. mdata] of Double;
    v_matrix = array [1 .. mcomp] of z_array;
    a_matrix = array [1 .. mcomp] of a0_array;

procedure SingValDecomp(
    var at: a_matrix; var vt: v_matrix; var z: z_array; m, n: Integer);


implementation

procedure SingValDecomp(
    var at: a_matrix; var vt: v_matrix; var z: z_array; m, n: Integer);

    {In order to figure out what this does, see p. 30, J. C. Nash,
     Compact Numerical Methods for Computers, John Wiley, New York, 1979.}

    var
        eps, tol, q, p, r, v, c, s, atji, vtji: Double;
        i, j, k, count, sweep: Integer;

begin
    {at[j][i] == A[i,j]}
    eps := 1.0e-10;
    tol := n*n*eps*eps;
    for i := 1 to n do begin
        for j := 1 to n do vt[j][i] {V[i,j]} := 0.0;
        vt[i][i] := 1.0;
    end;
    sweep := 0;
    repeat
{1}     count :=  (n*(n-1)) div 2;
        Inc(sweep);
{2}     for j := 1 to n-1 do begin
{3}         for k := j+1 to n do begin
{4}{5}   {      p := DotProduct(at[j],at[k],m);
                q := DotProduct(at[j],at[j],m);
                r := DotProduct(at[k],at[k],m);       }
                p := 0;
                for i := 1 to m do p := p + at[j][i]*at[k][i];
                q := 0;
                for i := 1 to m do q := q + at[j][i]*at[j][i];
                r := 0;
                for i := 1 to m do r := r + at[k][i]*at[k][i];
{}
{6}             if q < r then begin
{9}                 for i := 1 to m do begin
                        atji := at[j][i];
                        at[j][i] :=  at[k][i];
                        at[k][i] := -atji;
                    end;
{10}                for i := 1 to n do begin
                        vtji := vt[j][i];
                        vt[j][i] :=  vt[k][i];
                        vt[k][i] := -vtji;
                    end;
                end else
{7}             if (q*r < tol) OR ((p*p)/(q*r) < tol) then
{12}                Dec(count)
                else begin
{8}                 q := q - r;
                    v := Sqrt(4.0*p*p + q*q);
                    c := Sqrt((v+q)/(2.0*v));
                    s := p/(v*c);
{9}                 for i := 1 to m do begin
                        atji := at[j][i];
                        at[j][i] :=  c*atji + s*at[k][i];
                        at[k][i] := -s*atji + c*at[k][i];
                    end;
{10}                for i := 1 to n do begin
                        vtji := vt[j][i];
                        vt[j][i] :=  c*vtji + s*vt[k][i];
                        vt[k][i] := -s*vtji + c*vt[k][i];
                    end;
                end;
{13}        end;
{14}    end;
{15}until (count = 0) or (sweep = 6);
{16}for j := 1 to n do begin
{17}{18}
        q := 0;
        for i := 1 to m do q := q + at[j][i]*at[j][i];
{19}    q := Sqrt(q);
        z[j] := q;
{20}    if q > tol then
{21}        for i := 1 to m do at[j][i] := at[j][i]/q
        else
            z[j] := 0.0;
{22}end;
end;

end.
