PROGRAM VoiceG; {Copyright 1996, 1997 Paul Milenkovic ALL RIGHTS RESERVED}


USES StrBlock, StdIO, DSPTypes, DSPFast, LPC;

CONST
    out_max = 8192;
    out_mrk = 4096;
    mbuffr  = 1024;
    mpulse  =  512;
    mmpulse = -512;

VAR fname, extension: ASCIIZ_string;
    source_input: BOOLEAN;
    sample_ms: DOUBLE;
    data_scale, gain, fg, bg: SINGLE;
    out_buf: ARRAY [1 .. out_max] OF INTEGER;
    sig: ARRAY [mmpulse .. mbuffr] OF SINGLE;

PROCEDURE ScanCommandLineParameters;

    VAR i: INTEGER;
        pstr: ASCIIZ_string;

BEGIN
    fname     := '';
    extension := '';
    fg        := 0.050;
    bg        := 0.100;
    gain      := 1.0;

    FOR i := 1 TO ParamCount DO BEGIN
        pstr := ParamStr(i);
        UpdateSingleKeyword(fg,ParamStr(i),'/F:');
        UpdateSingleKeyword(bg,ParamStr(i),'/B:');
        UpdateSingleKeyword(gain,ParamStr(i),'/G:');
        UpdateStringKeyword(extension,ParamStr(i),'.');
        IF (fname = '')     AND
           (extension = '') AND
           (Copy(pstr,1,1) <> '/')
        THEN
            fname := pstr;
    END;

    IF bg <= 0 THEN
        bg := 2*fg;
END;


PROCEDURE ComputeCoef(VAR b0, a1, a2: SINGLE; ft, bt: SINGLE);

BEGIN
    a2 := Exp(-Pi*bt);
    a1 := -2*a2*Cos(2*Pi*ft);
    a2 := a2*a2;
    b0 := -1/a1;
END;


PROCEDURE InputVoiceSourcePulses;

    VAR vfile: TEXT;
        i, out_pos, out_dif, out_new, out_pre, out_nxt, ncoef, nc, p: INTEGER;
        t0, tf, tpulse, tpp: DOUBLE;
        apulse, fpulse, bpulse, dpulse, upulse: SINGLE;
        x0, x1, x2, y1, y2, b0, a1, a2: SINGLE;
{       pcoef: predictor;
        DC_scale, dmp: SINGLE;
 }

BEGIN
    Assign(vfile,fname);
    {$I-} Reset(vfile) {$I+};
    IF IOResult = 0 THEN BEGIN
        IF NOT source_input THEN
            StdCmd(Concat('ename=',fname));
        StdCmd('enable_break');
        t0 := StdDbl('start_ms?');
        tf := StdDbl('stop_ms?');
        tpulse := t0 - 20.0;
        IF EOF(vfile) THEN
            tpp := tf + 20.0
        ELSE
            Read(vfile,tpp);
        out_pos := 0;
        out_new := 0;
        StdCmd('');               {Initiate binary waveform I/O}
        REPEAT
            IF out_pos > out_mrk THEN BEGIN
                out_dif := out_pos - out_mrk;
                StdBlockWrite(out_buf,out_dif*Sizeof(INTEGER));
                IF StdNWrite = 0 THEN
                    out_pos := 0
                ELSE BEGIN
                    Move(
                        out_buf[Succ(out_dif)],out_buf[1],
                        out_mrk*Sizeof(INTEGER)
                    );
                    Dec(out_pos,out_dif);
                    Dec(out_new,out_dif);
                    IF out_new < 0 THEN out_new := 0;
                    t0 := t0 + out_dif * sample_ms;
                    FOR i := Succ(out_pos) TO out_max DO
                        out_buf[i] := 0;
                    out_pos := out_max;
                END;
            END ELSE BEGIN
                FOR i := Succ(out_pos) TO out_max DO
                    out_buf[i] := 0;
                out_pos := out_max;
            END;

            WHILE (tpulse <= tf) AND
                  (tpulse <= t0 + Pred(out_pos-(out_mrk DIV 2))*sample_ms)
            DO BEGIN
                IF tpulse >= t0 THEN BEGIN
                    out_pre := out_new;
                    out_new := Succ(Round((tpulse - t0)/sample_ms));
                    ComputeCoef(b0,a1,a2,fpulse*sample_ms,bpulse*sample_ms);
                    x2 := 0;
                    x1 := 0;
                    x0 := apulse*data_scale*gain{*DC_scale};
                    y2 := 0;
                    y1 := 0;
                    i := 0;
                    WHILE (out_new+i > out_pre) AND (i >= mmpulse) DO BEGIN
                        sig[i] := b0*(x0 - x2) - a1*y1 - a2*y2;
                        x2 := x1;
                        x1 := x0;
                        x0 := 0;
                        y2 := y1;
                        y1 := sig[i];
                        Dec(i);
                    END;
                    nc := 0;
                    IF tpp < tpulse + 20.0 THEN
                        out_nxt := Succ(Round((tpulse + (tpp-tpulse)*0.25 - t0)/sample_ms))
                    ELSE
                        out_nxt := out_new;
                    x2 := 0;
                    x1 := 0;
                    x0 := 0.0*apulse*data_scale*gain{*DC_scale};
                    y2 := 0;
                    y1 := 0;
                    WHILE (out_new+i < out_pos) AND (i < mbuffr) DO BEGIN
                        Inc(i);
                        (*
                        IF nc > 0 THEN BEGIN
                            IF (tpulse + i*sample_ms >
                                tpp - (tpp-tpulse)/2.5) AND
                               (tpulse + i*sample_ms <
                                tpp - (tpp-tpulse)/5)
                            THEN BEGIN
                                dmp := 1.0;
                                FOR p := 1 TO ncoef DO BEGIN
                                    dmp := dmp * 0.999;
                                    pcoef[p] := pcoef[p]*dmp;
                                END;
                            END;
                            IF i > Succ(out_nxt-out_new) THEN
                                sig[i] :=
                                -RevProduct(sig[i-1],pcoef[1],nc)
                            ELSE
                            IF i > 0 THEN
                                sig[i] := {(Random-0.5)*Sqrt(12)*0.005*data_scale*gain*DC_scale}
                                -RevProduct(sig[i-1],pcoef[1],nc)
                            ELSE
                                sig[i] := sig[i] -
                                RevProduct(sig[i-1],pcoef[1],nc);
                        END ELSE
                        IF i > Succ(out_nxt-out_new) THEN
                            sig[i] := 0
                        ELSE
                        IF i > 0 THEN
                            sig[i] := 0*(Random-0.5)*Sqrt(12)*0.005*data_scale*gain;

                        IF nc < ncoef THEN
                            Inc(nc);
                        *)
                        IF i >= 0 THEN BEGIN
                            sig[i] := b0*(x0 - x2) - a1*y1 - a2*y2;
                            x2 := x1;
                            x1 := x0;
                            x0 := 0;
                            y2 := y1;
                            y1 := sig[i];
                        END;
                        out_buf[out_new+i] := out_buf[out_new+i] +
                            Round(sig[i]);
                    END;
                END;
                tpulse := tpp;
                IF tpulse <= tf THEN BEGIN
                    Read(vfile,apulse,fpulse,bpulse);
                    ncoef := 0;
(*
                    pcoef[0] := 1.0;
                    WHILE (NOT EOLN(vfile)) AND (ncoef < mcoef) DO BEGIN
                        Inc(ncoef);
                        Read(vfile,pcoef[ncoef]);
                    END;
                    IF ncoef <= 1 THEN   {Allows processing .gpm files by ignoring filter index}
                        ncoef := 0;
                    IF ncoef > 0 THEN
                        Log2AreatoLPC(pcoef,ncoef);
                    DC_scale := pcoef[0];
                    FOR i := 1 TO ncoef DO
                        DC_scale := DC_scale + pcoef[i];
 *)                 Readln(vfile);
                END;
                IF EOF(vfile) THEN
                    tpp := tf + 20.0
                ELSE
                    Read(vfile,tpp);
            END;
        UNTIL out_pos = 0;

        StdSuspendWrite;
        Close(vfile);
    END;
END;

BEGIN
    StdSetErrorExit;
    ScanCommandLineParameters;

    StdCmd('leader');
    IF (fname = '') AND (extension <> '') THEN BEGIN
        StdCmd(Concat('extension=',extension));
        StdCmd('source');
        source_input := TRUE;
    END ELSE
        source_input := FALSE;
    StdCmd('output');
    IF fname = '' THEN BEGIN
        IF extension = '' THEN
            fname := StdTxt('ename?')
        ELSE
            fname := StdTxt('fname?');
    END;
    sample_ms := StdDbl('sample_ms?');

    IF (fname <> '') AND (sample_ms > 0) THEN BEGIN
        data_scale := 1.0/StdDbl('data_scale?');
        InputVoiceSourcePulses;
    END;
END.
